/*
	DIO demo for PCI-M512 SRAM battery backup card.

   OS       : MS-DOS
   CPU      : 80386 or higher
   Compiler : Borland C v3.1 (for DOS)

	This program doesn't support memory-management programs such as
	EMM386/QEMM386/DPMI... etc.

	This program shows the BT1 and BT2 batteries statues.	And outputs
	pattern to DO port, then read the DI port status.

	Users can connect the CON1 and CON2 with a 20-pin flat cable. And
	then you can see the DI 4~15 should be the same as DO 4~15,	while
	the DI 0~3 are for the battery status.
*/

#include "dos.h"
#include "PCIM512.h"

#define BOARD_ACT  1     // Board number to active

int TestDio(DWORD dwBoard, WORD wValue);
int DisplayBatteryStatus(WORD wStatus);

int main()
{
	DWORD dwRtn, dwBoards;  // Return code, Total-board found
	WORD  wBT;              // Battery Status at DI0~3

	clrscr(); // Clear screen

	/**************************************************/
	// Initialze the driver and find out PCI-M512 cards.
	dwRtn = PCIM512_DriverInit(&dwBoards);
	if ( dwRtn != 0 )
	{
		printf("Driver initialize error! Error Code=%d\n", (int)dwRtn);
		return 1;
	}
	if ( !dwBoards )
	{
		printf("Card not found!\n");
		return 1;
	}

	/*****************************/
	// Battery Status is at DI0~3
	PCIM512_ReadFromDi(BOARD_ACT, &wBT);

	printf("BT1 battery status is ");
	DisplayBatteryStatus(wBT);  // Show status at DI0~1

	printf("BT2 battery status is ");
	DisplayBatteryStatus(wBT >> 2); // Show status at DI2~3
	printf("\n");

	/****************************/
	// DI/DO testes
	printf("Please note, the DI 0~3 are for battery status.\n");
	TestDio(BOARD_ACT, 0xFFFF);
	TestDio(BOARD_ACT, 0xCCCC);
	TestDio(BOARD_ACT, 0xAAAA);
	TestDio(BOARD_ACT, 0x5555);
	TestDio(BOARD_ACT, 0x3333);
	TestDio(BOARD_ACT, 0x0000);

	/****************************/
	// Close driver before exit program
	PCIM512_DriverClose();

	getch();
	return 0;
}

/*****************************************/
// Test DI/DO
// dwBoard : Board number to active, base 1
// wValue  : 16-bit data to output for DO
int TestDio(DWORD dwBoard, WORD wValue)
{
	WORD wDI;
	PCIM512_WriteToDo(dwBoard, wValue);  // Digital Output
	delay(1);

	PCIM512_ReadFromDi(dwBoard, &wDI);   // Digital Input
	printf("DO=0x%04X, DI=0x%04X\n", wValue, wDI);
	return 0;
}

/**************************************/
// Display the battery status depends on wStatus
// wStatus : Battery status from DI0~1, or DI2~3
int DisplayBatteryStatus(WORD wStatus)
{
	if ( wStatus & 2 )      // battery is bad
		printf("bad!\n");
	else if ( wStatus & 1 ) // battery is low power
		printf("low!\n");
	else
		printf("good!\n");   // battery is good

	return 0;
}
