/*
	DI/DO demo for PISO-P16R16U

	Functions...
	1. Select board.
	2. Get Card-ID.
	3. Test DI/DO. (Should connect the DI with DO first).

*/


#include "PIO.H"

WORD wCardAddress, wTotalBoardsFound;

int SelectBoard(WORD wBoardNo);
DWORD TestDIO(WORD wCardAddress);
DWORD iCounts, iErrors;

int main()
{
	WORD wRtn, wIrqNo;
	WORD wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice;
	char c;
	WORD i;

	clrscr();

	/*************** Drivier Initialize **********************/
	wRtn = PIO_DriverInit(&wTotalBoardsFound,0x1800,0x0,0xFF);/* for PISO-16R16U*/
	if ( wTotalBoardsFound == 0 || wRtn != 0 )
	{
		printf("Driver initialize error! Error Code=%d\n", (int) wRtn);
		return 1;
	}
	printf("\nThere are %d PISO-P16R16U Card(s) in this PC",wTotalBoardsFound);

	/*********** Get Card's Information (I/O Address) *********/
	printf("\n\n-------------- The Configuration Space --------------");
	for(i=0;i<wTotalBoardsFound;i++)
	{
		PIO_GetConfigAddressSpace(i,&wCardAddress,&wIrqNo,&wSubVendor,&wSubDevice,&wSubAux,&wSlotBus,&wSlotDevice);

		printf("\nCard#%d: wBase=%x,wIrq=%x,subID=[%x,%x,%x],SlotID=[%x,%x]",i,wCardAddress,wIrqNo,wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice);

		printf(" --> ");
		ShowPioPiso(wSubVendor,wSubDevice,wSubAux);
	}

	/**** Select the Card#0 to active. ****/
	wRtn = SelectBoard(0);

	while (1)
	{
		printf("\nPlease select a function...\n");
		printf("1: Select a board to active.\n");
		printf("2: Display the Card-ID.\n");
		printf("3: DI/DO testes.\n");
		printf("Q: Exit the program.\n");
		c=getch();
		switch (c)
		{
		case '1': /************ Select Board *************/
			printf("Please select a board (0~%d) to active: ", wTotalBoardsFound -1);
			c = getch();
			if ( c >= '0' && c <= '9' )
				SelectBoard( c - '0' );
			else
				SelectBoard( 100 );
			c = ' ';
			break;

		case '2': /************* Get Card ID ************/
			printf("Card-ID is %d\n",(int) ( ~(inpb(wCardAddress +7) >> 2) & 0x0F) );
			break;

		case '3': /************ DI/DO testes ************/
			TestDIO(wCardAddress);
			break;

		case 'q': case 'Q':
			break;

		default:
			printf("Invalid function item %c\n", c);
			break;
		}
		if ( c == 'q' || c == 'Q' )
			break;
	}

	return 0;
}

int SelectBoard(WORD wBoardNo)
{
	WORD wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice;
	WORD wIrqNo;

	if ( wBoardNo >= wTotalBoardsFound )
	{
		printf("\nInvalid board number!\n");
		return ExceedBoardNumber;
	}

	/************ Get Card's Info (I/O Address) **************/
	PIO_GetConfigAddressSpace(wBoardNo,&wCardAddress,&wIrqNo,&wSubVendor,&wSubDevice,&wSubAux,&wSlotBus,&wSlotDevice);

	printf("\nCurrent selected board is Card#%d at address 0x%04X\n", wBoardNo, wCardAddress );

	if ( wCardAddress == 0 )
		return 100;

	/************ Enable all DI/DO **************/
	outpb(wCardAddress, 1);
	return 0;
}

int DIO1(WORD wCardAddress, WORD wValue)
{
	WORD wDI, Lo, Hi;

	/*********** Digital Output with wValue ***************/
	Lo = wValue & 0xFF;
	Hi = (wValue >> 8) & 0xFF;
	outpb( wCardAddress + 0xC0, Lo );  /* low-byte is at 0xC0  */
	outpb( wCardAddress + 0xC4, Hi );  /* High-byte is at 0xC4 */

	/****** delay a while for DO to ready ******/
	delay(100);

	/****** Digital Input *********/
	Lo = inpb( wCardAddress + 0xC0 );  /* Low-byte is at 0xC0  */
	Hi = inpb( wCardAddress + 0xC4 );  /* High-byte is at 0xC4 */
	wDI = (Hi << 8) | Lo;

	delay(300);
	iCounts++;

	/* Compare the DO and DI values */
	printf("DO=0x%04X, DI=0x%04X, ", wValue, wDI);
	if ( wDI != wValue )
	{
		printf("FAILED! (%04X)\n", wDI ^ wValue);
		iErrors++;
		return 1;
	}

	printf("OK! \n");
	return 0;
}

DWORD TestDIO(WORD wCardAddress)
{
	 int  i = 0;

	 iCounts = iErrors = 0;
	 //while ( !kbhit() )
	 while ( i < 100 )
	 {
		DIO1(wCardAddress, 0x0000);
		if (kbhit()) break;
		DIO1(wCardAddress, 0xFFFF);
		if (kbhit()) break;

		DIO1(wCardAddress, 0x0000);
		if (kbhit()) break;
		DIO1(wCardAddress, 0xAAAA);
		if (kbhit()) break;

		DIO1(wCardAddress, 0x0000);
		if (kbhit()) break;
		DIO1(wCardAddress, 0x5555);
		if (kbhit()) break;

		i += 3;
		//printf("TotalCounts=%ld, ErrorCounts=%ld\n\n", iCounts, iErrors);
	}

	DIO1(wCardAddress, 0x0000);
	 printf("TotalCounts=%ld, ErrorCounts=%ld\n", iCounts, iErrors);

	return iErrors;
}
