/* DEMO3 : Save EEPROM to file                                    */
/*                                                                */
/* -------------------------------------------------------------- */
#include "PIO.H"
#include <string.h>


WORD  wBase,wIrq;
WORD  wN10V[16],wP10V[16],w00mA[16],w20mA[16],EEP;

int main()
{
int   i,j,k;
WORD  wBoards,wRetVal,t1,t2,t3,t4,t5,t6;
WORD  wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice;
FILE  *fp;
char  filename[20];

clrscr();

/* step1 : find address-mapping of PIO/PISO cards                 */
wRetVal=PIO_DriverInit(&wBoards,0x80,0x0b,0x00);/*for PISO-DA2    */
printf("\n(1) Find %d PISO-DA2 in this PC",wBoards);
if ( wBoards==0 )  exit(0);

printf("\n\n-------------- The Configuration Space --------------");
for(i=0;i<wBoards;i++)
   {
   PIO_GetConfigAddressSpace(i,&wBase,&wIrq,&wSubVendor,&wSubDevice,&wSubAux,&wSlotBus,&wSlotDevice);

   printf("\nCard_%d: wBase=%x,wIrq=%x,subID=[%x,%x,%x],SlotID=[%x,%x]",i,wBase,wIrq,wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice);

   printf(" --> ");
   ShowPioPiso(wSubVendor,wSubDevice,wSubAux);
   }

PIO_GetConfigAddressSpace(0,&wBase,&wIrq,&t1,&t2,&t3,&t4,&t5);
						 /* select card_0 */
/* step2 : enable all D/I/O port                                  */
outportb(wBase,0x11);                            /* /RESET -> 1   */
outportb(wBase+2,0x1c);           /* AUX 4/3/2 are D/O, othes D/I */
outportb(wBase+3,0);              /* all D/O are Low              */

printf("\n\nCopy EEPROM data to a file"); 
printf("\n\n(2) Enter file name (for example, EEP.txt) : ");
scanf("%s",filename);

printf("\n\n(3) EEPROM data have been copied to file: %s",filename);
fp=fopen(filename,"wb+");

for (i=0; i<64;i++)
    {
    if (i<16)
       {
      EEP_READ(i,&j,&k);
      wN10V[i]=(j<<8)+k;
      fprintf(fp,"%5x",wN10V[i]);
      if (i==7) fprintf(fp,"\n");
      if (i==15) fprintf(fp,"\n");
       }
    if ((i>=16)&&(i<32))
       {
       EEP_READ(i,&j,&k);
       wP10V[i-16]=(j<<8)+k;
       fprintf(fp,"%5x",wP10V[i-16]);
       if (i==23) fprintf(fp,"\n");
       if (i==31) fprintf(fp,"\n");
       }
    if ((i>=32)&&(i<48))
       {
       EEP_READ(i,&j,&k);
       w00mA[i-32]=(j<<8)+k;
       fprintf(fp,"%5x",w00mA[i-32]);
       if (i==39) fprintf(fp,"\n");
       if (i==47) fprintf(fp,"\n");
       }
    if (i>=48)
       {
       EEP_READ(i,&j,&k);
       w20mA[i-48]=(j<<8)+k;
       fprintf(fp,"%5x",w20mA[i-48]);
       if (i==55) fprintf(fp,"\n");
       if (i==63) fprintf(fp,"\n");
       }
    }
fprintf(fp,"========================================\n");
fprintf(fp,"File name : %s",filename);
printf("\n\nPress any key to exit!");
getch();

fclose(fp);

outportb(wBase+5,0);                     /* disable all interrupt */
outportb(wBase+3,0);                     /* all D/O are Low       */
outportb(wBase+2,0);                     /* all AUX as D/I        */
PIO_DriverClose();
}

/* -------------------------------------------------------------- */
/* AUX2= D/O = CS of EEP            AUX3= D/O = SK of EEP         */
/* AUX4= D/O = DI of EEP            AUX5= D/I = DO of EEP         */
/* -------------------------------------------------------------- */
COMM()                                  /* send start 2-bit (0,1) */
{
EEP=0x04; outportb(wBase+3,EEP);        /* CS=H */
EEP=0x0C; outportb(wBase+3,EEP);        /* SK=H */
					/* DI=0 */
EEP=0x04; outportb(wBase+3,EEP);        /* SK=L */
EEP=0x14; outportb(wBase+3,EEP);        /* DI=1 */
EEP=0x1c; outportb(wBase+3,EEP);        /* SK=H */
}

/* -------------------------------------------------------------- */
WR_BYTE(int val)                           /* send the next 8-bit */
{
int i,j;

for (i=0; i<8; i++)
    {
    EEP=EEP&0x14; outportb(wBase+3,EEP);   /* SK=L */
    j=val&0x80; val=val<<1;
    if (j==0)                              /* DI=0 */
       {
	EEP=EEP&0x0C; outportb(wBase+3,EEP);
       }
    else                                   /* DI=1 */
       {
	EEP=EEP|0x10; outportb(wBase+3,EEP);
       }
    EEP=EEP|0x08; outportb(wBase+3,EEP);   // SK=H
    }
}

// --------------------------------------------------------------
RD_BYTE()                                  // read the next 8-bit
{
int i,j,val,k;

val=0x80; k=0;
for (i=0; i<8; i++)
    {
     EEP=EEP&0x14; outportb(wBase+3,EEP);   // SK=L
     j=(inportb(wBase+7))&0x20;
     if (j!=0) k+=val;                      // DO=1
     EEP=EEP|0x08; outportb(wBase+3,EEP);   // SK=H
     val=val>>1;
    }
return(k);
}

/* -------------------------------------------------------------- */
EEP_READ(int ADDR, int *HI, int *LO) /* read 16-bit data from EEP */
{
 int i,j,k;

 COMM();
 ADDR=ADDR|0x80; WR_BYTE(ADDR);          /* read command & address */
 EEP=EEP&0x0C; outportb(wBase+3,EEP);    /* DI=0 */
 EEP=EEP&0x14; outportb(wBase+3,EEP);    /* SK=L */
 EEP=EEP|0x08; outportb(wBase+3,EEP);    /* SK=H */
 EEP=EEP&0x14; outportb(wBase+3,EEP);    /* SK=L */
 (*HI)=RD_BYTE();                        /* read high 8-bit data   */
 (*LO)=RD_BYTE();                        /* read low  8-bit data   */
 EEP=EEP&0x14; outportb(wBase+3,EEP);    /* SK=L */
 EEP=EEP&0x18; outportb(wBase+3,EEP);    /* CS=L */
}
