/* -------------------------------------------------------------- */
/* DEMO 5 : Software calibration (manual tuning)                  */
/* Note   : User need one Current meter & one Voltage meter       */
/*          to measure D/A output(current/voltage) value. By the  */
/*          result user decide increment or decrement Hex value   */
/*          that be sent to D/A converter. Finally, all Hex value */
/*          will store to EEPROM                                  */
/* Step  1: Run DEMO5.EXE                                        */
/* -------------------------------------------------------------- */
#include "PIO.H"


#define PreSetN10V      4095
#define PreSetP10V      4095
#define PreSetN5V       4095
#define PreSetP5V       4095
#define PreSet00mA      0
#define PreSet20mA      4095
#define PreSet4mA       0
#define PreSet420mA     4095



int   iComPort,iChksum,j,chan,temp;
WORD  wBase,wIrq,EEP;
WORD  wDelay1=2000;
WORD  wN10V[16],wP10V[16],wN5V[16],wP5V[16],w00mA[16],w20mA[16],w4mA[16],w420mA[16];
float rr[16];
char  c,cc,str1[40],str2[40],str3[40];

long int lBaudRate;

int  save_neg_10v_to_eeprom(void);
int  save_pos_10v_to_eeprom(void);
int  save_neg_5v_to_eeprom(void);
int  save_pos_5v_to_eeprom(void);
int  save_00mA_to_eeprom(void);
int  save_20mA_to_eeprom(void);
int  save_4mA_to_eeprom(void);
int  save_420mA_to_eeprom(void);



int  cal_N10V(void);
int  cal_B10V(void);
int  cal_N5V(void);
int  cal_B5V(void);
int  cal_00mA(void);
int  cal_20mA(void);
int  cal_4mA(void);
int  cal_420mA(void);



int  PISO_DA2_AnalogOutput(int, int);

int main()
{
int   i,h,l;
WORD  wBoards,wRetVal,t1,t2,t3,t4,t5,t6;
WORD  wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice;
WORD  k,wStart,wBegin1,wBegin2,LH1,LH2,LL1,LL2;
char  str1[80];


clrscr();

/* Step1 : Find address-mapping of PIO/PISO cards                 */
wRetVal=PIO_DriverInit(&wBoards,0x80,0x0b,0x00);  /* for PISO-DA2 */
printf("\n(1) There are %d PISO-DA2 Cards in this PC",wBoards);
if ( wBoards==0 )  exit(0);

printf("\n\n-------------- The Configuration Space --------------");
for(i=0;i<wBoards;i++)
   {
   PIO_GetConfigAddressSpace(i,&wBase,&wIrq,&wSubVendor,&wSubDevice,&wSubAux,&wSlotBus,&wSlotDevice);

   printf("\nCard_%d: wBase=%x,wIrq=%x,subID=[%x,%x,%x],SlotID=[%x,%x]",i,wBase,wIrq,wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice);

   printf(" --> ");
   ShowPioPiso(wSubVendor,wSubDevice,wSubAux);
   }

PIO_GetConfigAddressSpace(0,&wBase,&wIrq,&t1,&t2,&t3,&t4,&t5);
						  /* select card_0 */
/* step2 : enable all D/I/O port                                  */
outportb(wBase,0x11);                             /* /RESET -> 1  */

outportb(wBase+2,0x1C);           /* AUX 4/3/2 are D/O, othes D/I */
outportb(wBase+3,0);              /* all D/O are Low              */

hand:
clrscr();
printf("\n\nDEMO.5 D/A Calibrator test");
printf("\n(1) Channel 1");
printf("\n(2) Channel 2");
printf("\n(3) Quit");


printf("\nSelect (1-3): ");
    c=getche();
    switch(c)
     {
	 case '1': LH1=0xc0,LL1=0xc4,chan=1;
		   break;
	 case '2': LH2=0xc8,LL2=0xc8,chan=2;
		   break;
	 case '3':
	 case 'Q':
	 case 'q': return;

    default : return;
   }

   if (chan==1) temp=0;
   else if (chan==2) temp=8;

for(;;)
   {
   clrscr();
   gotoxy(1,1);
   printf("Channel %2d",chan);
   printf("\n***************************************");
   printf("\n*            D/A Calibrator           *");
   printf("\n*-------------------------------------*");
   printf("\n* (1) 0~+10V        (2) -10~+10V      *");
   printf("\n* (3) 0~ +5V        (4) -5 ~ +5V      *");
   printf("\n* (5)  0mA          (6) 20mA(0~20mA)  *");
   printf("\n* (7)  4mA          (8) 20mA(4~20mA)  *");
   printf("\n* (9)  ALL          (A) QUIT          *");
   printf("\n***************************************");
   printf("\n\nSelect (1-9):");cc=getche();
   printf("\n\nWait ....");

   switch (cc)
      {
      case '1': cal_N10V();
		break;
      case '2': cal_B10V();
		break;
      case '3': cal_N5V();
		break;
      case '4': cal_B5V();
		break;
      case '5': cal_00mA();
		break;
      case '6': cal_20mA();
		break;
      case '7': cal_4mA();
		break;
      case '8': cal_420mA();
		break;
      case '9': cal_N10V();
		cal_B10V();
		cal_N5V();
		cal_B5V();
		cal_00mA();
		cal_20mA();
		cal_4mA();
		cal_420mA();
		break;
      case 'a': goto hand;
      default : break;
      }


   clrscr();
   gotoxy(1,8);

   for (i=0; i<1; i++)
       {
	EEP_READ(i,&j,&k);
	l=wN10V[i]&0xff;
	h=(wN10V[i]>>8)&0x0f;
	printf("\nREAD from EEPROM Channel=%2d, N10V=>[MSB=%2x,LSB=%2x]",chan,h,l);
       }
   for (i=1; i<2; i++)
       {
	EEP_READ(i,&j,&k);
	l=wP10V[i]&0xff;
	h=(wP10V[i]>>8)&0x0f;
	printf("\nREAD from EEPROM Channel=%2d, P10V=>[MSB=%2x,LSB=%2x]",chan,h,l);
       }
  for (i=2; i<3; i++)
       {
	EEP_READ(i,&j,&k);
	l=wN5V[i]&0xff;
	h=(wN5V[i]>>8)&0x0f;
	printf("\nREAD from EEPROM Channel=%2d,  N5V=>[MSB=%2x,LSB=%2x]",chan,h,l);
       }
  for (i=3; i<4; i++)
       {
	EEP_READ(i,&j,&k);
	l=wP5V[i]&0xff;
	h=(wP5V[i]>>8)&0x0f;
	printf("\nREAD from EEPROM Channel=%2d,  P5V=>[MSB=%2x,LSB=%2x]",chan,h,l);
       }
  for (i=4; i<5; i++)
       {
	EEP_READ(i,&j,&k);
	l=w00mA[i]&0xff;
	h=(w00mA[i]>>8)&0x0f;
	printf("\nREAD from EEPROM Channel=%2d, 00mA=>[MSB=%2x,LSB=%2x]",chan,h,l);
       }
  for (i=5; i<6; i++)
       {
	EEP_READ(i,&j,&k);
	l=w20mA[i]&0xff;
	h=(w20mA[i]>>8)&0x0f;
	printf("\nREAD from EEPROm Channel=%2d, 20mA=>[MSB=%2x,LSB=%2x]",chan,h,l);
       }

  for (i=6; i<7; i++)
       {
	EEP_READ(i,&j,&k);
	l=w4mA[i]&0xff;
	h=(w4mA[i]>>8)&0x0f;
	printf("\nREAD from EEPROm Channel=%2d,  4mA=>[MSB=%2x,LSB=%2x]",chan,h,l);
       }

  for (i=7; i<8; i++)
       {
	EEP_READ(i,&j,&k);
	l=w420mA[i]&0xff;
	h=(w420mA[i]>>8)&0x0f;
	printf("\nREAD from EEPROm Channel=%2d,420mA=>[MSB=%2x,LSB=%2x]",chan,h,l);
       }

   getch();
   gotoxy(1,15);

 }

outportb(wBase+3,0);                           /* all D/O are Low */
outportb(wBase+2,0);                           /* all AUX as D/I  */
PIO_DriverClose();
}

/* -------------------------------------------------------------- */
/* Output to DA channel                                           */
/* channel : 0 - 2                                                */
/* value   : 0 - 4096 (0xFFF)                                     */
/* -------------------------------------------------------------- */
int PISO_DA2_AnalogOutput(int channel, int da_value)
{
  WORD MSB,LSB,LH,LL;

  if(channel==1)     {LH=0xc0,LL=0xc4;}

  else if(channel==2){LH=0xc8,LL=0xcc;}

  MSB=da_value/256;
  LSB=da_value%256;

  outportb(wBase+LH, MSB);
  outportb(wBase+LL, LSB);
  return;
}

// --------------------------------------------------------------
save_neg_10v_to_eeprom()
{
WORD k,high,low,j;

EEP_WR_EN();
    low=wN10V[0]&0xff;
    high=(wN10V[0]>>8)&0x0f;
    EEP_WRITE(0+temp,high,low);
}

// --------------------------------------------------------------
save_pos_10v_to_eeprom()
{
WORD k,high,low;
EEP_WR_EN();
    low=wP10V[1]&0xff;
    high=(wP10V[1]>>8)&0x0f;
    EEP_WRITE(1+temp,high,low);
}

// --------------------------------------------------------------
save_neg_5v_to_eeprom()
{
WORD k,high,low;

EEP_WR_EN();
    low=wN5V[2]&0xff;
    high=(wN5V[2]>>8)&0x0f;
    EEP_WRITE(2+temp,high,low);
}

// --------------------------------------------------------------
save_pos_5v_to_eeprom()
{
WORD k,high,low;

EEP_WR_EN();
    low=wP5V[3]&0xff;
    high=(wP5V[3]>>8)&0x0f;
    EEP_WRITE(3+temp,high,low);
}


// --------------------------------------------------------------
save_00mA_to_eeprom()
{
WORD k,high,low;

EEP_WR_EN();
    low=w00mA[4]&0xff;
    high=(w00mA[4]>>8)&0x0f;
    EEP_WRITE(4+temp,high,low);
}

// --------------------------------------------------------------
save_20mA_to_eeprom()
{
WORD k,high,low;

EEP_WR_EN();
    low=w20mA[5]&0xff;
    high=(w20mA[5]>>8)&0x0f;
    EEP_WRITE(5+temp,high,low);
}


// --------------------------------------------------------------
save_4mA_to_eeprom()
{
WORD k,high,low;

EEP_WR_EN();
    low=w4mA[6]&0xff;
    high=(w4mA[6]>>8)&0x0f;
    EEP_WRITE(6+temp,high,low);
}

// --------------------------------------------------------------
save_420mA_to_eeprom()
{
WORD k,high,low;

EEP_WR_EN();
    low=w420mA[7]&0xff;
    high=(w420mA[7]>>8)&0x0f;
    EEP_WRITE(7+temp,high,low);
}


// --------------------------------------------------------------
delay_ms(int t)
{
int   i,j,k;
float p,q,r;

for (i=0; i<t; i++)
for (k=0; k<wDelay1; k++)
    {
    p=i+t;
    q=i+k;
    r=p*q;
    }
}

// --------------------------------------------------------------
// AUX2= D/O = CS of EEP            AUX3= D/O = SK of EEP
// AUX4= D/O = DI of EEP            AUX5= D/I = DO of EEP
// --------------------------------------------------------------

COMM()                                  // send start 2-bit (0,1)
{
EEP=0x04; outportb(wBase+3,EEP);        // CS=H
EEP=0x0C; outportb(wBase+3,EEP);        // SK=H
					// DI=0
EEP=0x04; outportb(wBase+3,EEP);        // SK=L
EEP=0x14; outportb(wBase+3,EEP);        // DI=1
EEP=0x1c; outportb(wBase+3,EEP);        // SK=H
}

// --------------------------------------------------------------
WR_BYTE(int val)                           // send the next 8-bit
{
int i,j;

for (i=0; i<8; i++)
    {
    EEP=EEP&0x14; outportb(wBase+3,EEP);   // SK=L
    j=val&0x80; val=val<<1;
    if (j==0)                              // DI=0
       {
       EEP=EEP&0x0C; outportb(wBase+3,EEP);
       }
    else                                   // DI=1
       {
       EEP=EEP|0x10; outportb(wBase+3,EEP);
       }
    EEP=EEP|0x08; outportb(wBase+3,EEP);   // SK=H
    }
}

// --------------------------------------------------------------
RD_BYTE()                                  // read the next 8-bit
{
int i,j,val,k;

val=0x80; k=0;
for (i=0; i<8; i++)
    {
    EEP=EEP&0x14; outportb(wBase+3,EEP);   // SK=L
    j=(inportb(wBase+7))&0x20;
    if (j!=0) k+=val;                      // DO=1
    EEP=EEP|0x08; outportb(wBase+3,EEP);   // SK=H
    val=val>>1;
    }
return(k);
}

// --------------------------------------------------------------
EEP_WR_EN()                               /* write Enable */
{
COMM();
WR_BYTE(0x30);                            /* write enable command */
EEP=EEP&0x14; outportb(wBase+3,EEP);      /* SK=L */
EEP=EEP&0x18; outportb(wBase+3,EEP);      /* CS=L */
EEP=EEP&0x0C; outportb(wBase+3,EEP);      /* DI=0 */
}

/* -------------------------------------------------------------- */
EEP_WR_DIS()                             /* write Disable */
{
COMM();
WR_BYTE(0);                              /* write disable command */
EEP=EEP&0x14; outportb(wBase+3,EEP);     /* SK=L */
EEP=EEP&0x18; outportb(wBase+3,EEP);     /* CS=L */
EEP=EEP&0x0C; outportb(wBase+3,EEP);     /* DI=0 */
}

/* -------------------------------------------------------------- */
EEP_WRITE(int ADDR, int HI, int LO)   /* write 16-bit data to EEP */
{
int i,j,k;

COMM();
ADDR=ADDR|0x40; WR_BYTE(ADDR);        /* write command & address  */
WR_BYTE(HI);                          /* high 8-bit data          */
WR_BYTE(LO);                          /* low  8-bit data          */

EEP=EEP&0x14; outportb(wBase+3,EEP);  /* SK=L */
EEP=EEP&0x18; outportb(wBase+3,EEP);  /* CS=L */
EEP=EEP&0x0C; outportb(wBase+3,EEP);  /* DI=0 */

for (i=0; i<10; i++)                  /* delay>250 ns (for EEP)   */
{
EEP=EEP|0x08; outportb(wBase+3,EEP);  /* SK=H */
EEP=EEP&0x14; outportb(wBase+3,EEP);  /* SK=L */
}

EEP=EEP|0x04; outportb(wBase+3,EEP);  /* CS=H */

for (k=0; k<10; k++)                  /* wait until write finish  */
for (i=0; i<30000; i++)
    {
    EEP=EEP|0x08; outportb(wBase+3,EEP);/* SK=H */
    j=inportb(wBase+7)&0x20;
    EEP=EEP&0x14; outportb(wBase+3,EEP);/* SK=L */
    if (j!=0) return(0);                /* write is finished      */
    }
EEP=EEP&0x18; outportb(wBase+3,EEP);    /* CS=L */
return(1);                              /* error --> time out ?   */
}

/* -------------------------------------------------------------- */
EEP_READ(int ADDR, int *HI, int *LO) /* read 16-bit data from EEP */
{
int i,j,k;

COMM();
ADDR=ADDR|0x80; WR_BYTE(ADDR);          /* read command & address */
EEP=EEP&0x0C; outportb(wBase+3,EEP);    /* DI=0 */
EEP=EEP&0x14; outportb(wBase+3,EEP);    /* SK=L */
EEP=EEP|0x08; outportb(wBase+3,EEP);    /* SK=H */
EEP=EEP&0x14; outportb(wBase+3,EEP);    /* SK=L */
(*HI)=RD_BYTE();                        /* read high 8-bit data   */
(*LO)=RD_BYTE();                        /* read low  8-bit data   */
EEP=EEP&0x14; outportb(wBase+3,EEP);    /* SK=L */
EEP=EEP&0x18; outportb(wBase+3,EEP);    /* CS=L */
}

// --------------------------------------------------------------
cal_N10V()
{
int   i,j;
char  ch=0;

j=PreSetN10V;
    clrscr();
    PISO_DA2_AnalogOutput(chan,j);
    delay_ms(500);
    printf(" '7' => +10  , '9' => +1\n");
    printf(" '1' => -10  , '3' => -1\n");
    printf(" '0' => O.K    \nPlease change Jumper to -10V position \n\n");

    do
       {
       printf("\r0 ~ +10v Calibration -> chan_%2d, wN10V = %4x hex=%4d",chan,j,j);
       ch=getch();
       if (ch=='9') j=j+1;
       if (ch=='3') j=j-1;
       if (ch=='7') j=j+10;
       if (ch=='1') j=j-10;
       PISO_DA2_AnalogOutput(chan,j);
       delay_ms(500);
       }
    while(ch!='0');
    wN10V[0]=j;

printf("\n\nWrite data to EEPROM, wait .....");
delay(10);

save_neg_10v_to_eeprom();
}


// --------------------------------------------------------------
cal_B10V()
{
int   i,j;
char  ch=0;

j=PreSetP10V;
    clrscr();
    PISO_DA2_AnalogOutput(chan,j);
    delay_ms(500);
    printf(" '7' => +10  , '9' => +1\n");
    printf(" '1' => -10  , '3' => -1\n");
    printf(" '0' => O.K    \nPlease change Jumper to -10V position \n\n");

    do
       {
       printf("\r-10~+10V Calibration -> chan_%2d, wP10V = %4x hex=%4d",chan,j,j);
       ch=getch();
       if (ch=='9') j=j+1;
       if (ch=='3') j=j-1;
       if (ch=='7') j=j+10;
       if (ch=='1') j=j-10;
       PISO_DA2_AnalogOutput(chan,j);
       delay_ms(500);
       }
    while(ch!='0');
    wP10V[1]=j;

printf("\n\nWrite data to EEPROM, wait .....");
delay(10);
save_pos_10v_to_eeprom();
}


// --------------------------------------------------------------
cal_N5V()
{
int   i,j;
char  ch=0;

j=PreSetN5V;
    clrscr();
    PISO_DA2_AnalogOutput(chan,j);
    delay_ms(500);
    printf(" '7' => +10  , '9' => +1\n");
    printf(" '1' => -10  , '3' => -1\n");
    printf(" '0' => O.K    \nPlease change Jumper to -5V position \n\n");
    do
       {
       printf("\r0~+5v Calibration -> chan_%2d, wN10V = %4x hex=%4d",chan,j,j);
       ch=getch();
       if (ch=='9') j=j+1;
       if (ch=='3') j=j-1;
       if (ch=='7') j=j+10;
       if (ch=='1') j=j-10;
       PISO_DA2_AnalogOutput(chan,j);
       delay_ms(500);
       }
    while(ch!='0');
    wN5V[2]=j;

printf("\n\nWrite data to EEPROM, wait .....");
save_neg_5v_to_eeprom();
}

// --------------------------------------------------------------
cal_B5V()
{
int   i,j;
char  ch=0;


j=PreSetP5V;

    clrscr();
    PISO_DA2_AnalogOutput(chan,j);
    delay_ms(500);
    printf(" '7' => +10  , '9' => +1\n");
    printf(" '1' => -10  , '3' => -1\n");
    printf(" '0' => O.K    \nPlease change Jumper to -5V position \n\n");
    do
       {
       printf("\r-5+5V Calibration -> chan_%2d, wP10V = %4x hex=%4d",chan,j,j);
       ch=getch();
       if (ch=='9') j=j+1;
       if (ch=='3') j=j-1;
       if (ch=='7') j=j+10;
       if (ch=='1') j=j-10;
       PISO_DA2_AnalogOutput(chan,j);
       delay_ms(500);
       }
    while(ch!='0');
    wP5V[3]=j;
printf("\n\nWrite data to EEPROM, wait .....");
save_pos_5v_to_eeprom();
}


// --------------------------------------------------------------

cal_00mA()
{
int   i,j;
char  ch=0;


j=PreSet00mA;
    clrscr();
    PISO_DA2_AnalogOutput(chan,j);
    delay_ms(500);
    printf(" '7' => +10  , '9' => +1\n");
    printf(" '1' => -10  , '3' => -1\n");
    printf(" '0' => O.K\n");
    do
       {
       printf("\r00mA Calibration -> chan_%2d, w00mA = %4x hex=%4d",chan,j,j);
       ch=getch();
       if (ch=='9') j=j+1;
       if (ch=='3') j=j-1;
       if (ch=='7') j=j+10;
       if (ch=='1') j=j-10;
       PISO_DA2_AnalogOutput(chan,j);
       delay_ms(500);
       }
    while(ch!='0');
    w00mA[4]=j;
printf("\n\nWrite data to EEPROM, wait .....");
save_00mA_to_eeprom();
}

// --------------------------------------------------------------
cal_20mA()
{
int   i,j;
char  ch=0;

j=PreSet20mA;
    clrscr();
    PISO_DA2_AnalogOutput(chan,j);
    delay_ms(500);
    printf(" '7' => +10  , '9' => +1\n");
    printf(" '1' => -10  , '3' => -1\n");
    printf(" '0' => O.K\n");
    do
       {
       printf("\r20mA Calibration -> chan_%2d, w20mA = %4x hex=%4d",chan,j,4095-j);
       ch=getch();
       if (ch=='9') j=j+1;
       if (ch=='3') j=j-1;
       if (ch=='7') j=j+10;
       if (ch=='1') j=j-10;
       PISO_DA2_AnalogOutput(chan,j);
       delay_ms(500);
       }
    while(ch!='0');
    w20mA[5]=j;
printf("\n\nWrite data to EEPROM, wait .....");
save_20mA_to_eeprom();
}

// --------------------------------------------------------------
cal_4mA()
{
int   j;
char  ch=0;

j=PreSet4mA;
    clrscr();
    PISO_DA2_AnalogOutput(chan,j);
    delay_ms(500);
    printf(" '7' => +10  , '9' => +1\n");
    printf(" '1' => -10  , '3' => -1\n");
    printf(" '0' => O.K\n");
    do
       {
       printf("\r4mA Calibration -> chan_%2d, w4mA = %4x hex=%4d",chan,j,j);
       ch=getch();
       if (ch=='9') j=j+1;
       if (ch=='3') j=j-1;
       if (ch=='7') j=j+10;
       if (ch=='1') j=j-10;
       PISO_DA2_AnalogOutput(chan,j);
       delay_ms(500);
       }
    while(ch!='0');
    w4mA[6]=j;

printf("\n\nWrite data to EEPROM, wait .....");
save_4mA_to_eeprom();
}

// --------------------------------------------------------------
cal_420mA()
{
int   j;
char  ch=0;

j=PreSet420mA;
    clrscr();
    PISO_DA2_AnalogOutput(chan,j);
    delay_ms(500);
    printf(" '7' => +10  , '9' => +1\n");
    printf(" '1' => -10  , '3' => -1\n");
    printf(" '0' => O.K\n");
    do
       {
       printf("\r420mA Calibration -> chan_%2d, w420mA = %4x  hex=%4d",chan,j,4095-j);
       ch=getch();
       if (ch=='9') j=j+1;
       if (ch=='3') j=j-1;
       if (ch=='7') j=j+10;
       if (ch=='1') j=j-10;
       PISO_DA2_AnalogOutput(chan,j);
       delay_ms(500);
       }
    while(ch!='0');
    w420mA[7]=j;

printf("\n\nWrite data to EEPROM, wait .....");
save_420mA_to_eeprom();
}
