#include  <stdio.h>
#include  <stdlib.h>
#include  <dos.h>
#include  <conio.h>
#include  <math.h>

#define   WORD	    unsigned short
#define   DWORD    unsigned long
#define   UCHAR    unsigned char

// Error Codes
#define   PCIM512_NoError 		               0
#define   PCIM512_FindBoardError             9
#define   PCIM512_ExceedBoardNumber	         10
#define   PCIM512_Timeout 		               23
#define   PCIM512_AddressNotAllocatedByBIOS  30
#define   PCIM512_MemoryOffsetOutOfRange     31

// **************
// Test functions

// fA     : [IN] floating value
// fB     : [IN] floating value
// return : floating value of (fA - fB)
float PCIM512_FloatSub2(float fA, float fB);

// nA     : [IN] short integer value
// nB     : [IN] short integer value
// return : short integer value of (nA - nB)
short PCIM512_ShortSub2(short nA, short nB);

// ****************
// Driver functions

// Initialize the driver and find out the PCI-M512 cards
// dwBoards : [OUT] Stores the number of Total-Boards found by driver
// return   : 0 for success, others for error code
DWORD PCIM512_DriverInit(DWORD *dwBoards);

// Close the driver
// return   : 0 for success, others for error code
DWORD PCIM512_DriverClose(void);

// Get the driver version number
// wDriverVersion : [OUT] Stores the driver version number
// return         : 0 for success, others for error code
DWORD PCIM512_GetDriverVersion(WORD *wDriverVersion);

// Get board's PCI Hardware ID
// dwBoardNo     : [IN] Specify the board number to access
// dwVendorId    : [OUT] Returns the Vendor-ID     (0x10b5)
// dwDeviceId    : [OUT] Returns the Device-ID     (0x9050)
// dwSubVendorId : [OUT] Returns the Sub-Vendor-ID (0x2129)
// dwSubDeviceId : [OUT] Returns the Sub-Device-ID (0x0512)
// return        : 0 for success, others for error code
DWORD PCIM512_ReadBoardId(DWORD dwBoardNo, DWORD *dwVendorId,DWORD *dwDeviceId,DWORD *dwSubVendorId,DWORD *dwSubDeviceId);


// ****************************
// SRAM Memory access functions

// Write a byte(8-bit) to SRAM memory
// dwBoardNo : [IN] Board number to access, base 1
// dwOffset  : [IN] Memory offset between 0 to SRAM_Size -1
// Data      : [IN] 8-bit data
// return    : 0 for success, others for error code
DWORD PCIM512_WriteSramByte( DWORD dwBoardNo, DWORD dwOffset, UCHAR Data);

// Write a word(16-bit) to SRAM memory
// dwBoardNo : [IN] Board number to access, base 1
// dwOffset  : [IN] Memory offset between 0 to SRAM_Size -1
// Data      : [IN] 16-bit data
// return    : 0 for success, others for error code
DWORD PCIM512_WriteSramWord( DWORD dwBoardNo, DWORD dwOffset, WORD Data);

// Write a double-word(32-bit) to SRAM memory
// dwBoardNo : [IN] Board number to access, base 1
// dwOffset  : [IN] Memory offset between 0 to SRAM_Size -1
// Data      : [IN] 32-bit data
// return    : 0 for success, others for error code
DWORD PCIM512_WriteSramDword(DWORD dwBoardNo, DWORD dwOffset, DWORD Data);

// Read a byte(8-bit) from SRAM memory
// dwBoardNo : [IN] Board number to access, base 1
// dwOffset  : [IN] Memory offset between 0 to SRAM_Size -1
// Data      : [OUT] Stores a 8-bit data after calling this function
// return    : 0 for success, others for error code
DWORD PCIM512_ReadSramByte( DWORD dwBoardNo, DWORD dwOffset, UCHAR *Data);

// Read a word(16-bit) from SRAM memory
// dwBoardNo : [IN] Board number to access, base 1
// dwOffset  : [IN] Memory offset between 0 to SRAM_Size -1
// Data      : [OUT] Stores a 16-bit data after calling this function
// return    : 0 for success, others for error code
DWORD PCIM512_ReadSramWord( DWORD dwBoardNo, DWORD dwOffset, WORD *Data);

// Read a double-word(32-bit) from SRAM memory
// dwBoardNo : [IN] Board number to access, base 1
// dwOffset  : [IN] Memory offset between 0 to SRAM_Size -1
// Data      : [OUT] Stores a 32-bit data after calling this function
// return    : 0 for success, others for error code
DWORD PCIM512_ReadSramDword(DWORD dwBoardNo, DWORD dwOffset, DWORD *Data);

// **********************
// DI/DO access functions

// Write a word(16-bit) to the Digital-Output port
// dwBoardNo : [IN] Board number to access, base 1
// Data      : [IN] 16-bit data to output
// return    : 0 for success, others for error code
DWORD PCIM512_WriteToDo( DWORD dwBoardNo, WORD Data);

// Read a word(16-bit) from the Digital-Input port.
// Please note, that the DI0~DI3 are for battery status.
// dwBoardNo : [IN] Board number to access, base 1
// Data      : [OUT] Stores 16-bit data after calling this function.
// return    : 0 for success, others for error code
DWORD PCIM512_ReadFromDi(DWORD dwBoardNo, WORD *Data);

