/* demo 5 : Four interrupt source                                 */
/*          P2C0  : initial Low  , active High                    */
/*          P5C0  : initial High , active Low                     */
/*          P8C0  : initial Low  , active High                    */
/*          P11C0 : initial High , active Low                     */
/* step 1 : run DEMO5.EXE                                         */
/* -------------------------------------------------------------- */

#include "PIO.H"

#define A1_8259 0x20
#define A2_8259 0xA0
#define EOI     0x20

WORD init();
WORD wBase,wIrq;

static void interrupt irq_service();
int irqmask,now_int_state,new_int_state,invert,int_c,int_num;
int CNT_L1,CNT_L2,CNT_L3,CNT_L4;
int CNT_H1,CNT_H2,CNT_H3,CNT_H4;

int main()
{
int  i,j;
WORD wBoards,wRetVal,t1,t2,t3,t4,t5,t6;
WORD wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice;
char c;

clrscr();

/* step1 : find address-mapping of PIO/PISO cards                 */
wRetVal=PIO_DriverInit(&wBoards,0x80,0x01,0x10);   /* for PIO-D96 */
printf("\n(1) Threr are %d PIO-D96 Cards in this PC",wBoards);
if ( wBoards==0 ) exit(0);

printf("\n\n-------------- The Configuration Space --------------");
for(i=0;i<wBoards;i++)
   {
   PIO_GetConfigAddressSpace(i,&wBase,&wIrq,&wSubVendor,&wSubDevice,&wSubAux,&wSlotBus,&wSlotDevice);

   printf("\nCard_%d: wBase=%x,wIrq=%x,subID=[%x,%x,%x],SlotID=[%x,%x]",i,wBase,wIrq,wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice);

   printf(" --> ");
   ShowPioPiso(wSubVendor,wSubDevice,wSubAux);
   }

PIO_GetConfigAddressSpace(0,&wBase,&wIrq,&t1,&t2,&t3,&t4,&t5);
                                                 /* select card_0 */
/* step2 : enable all D/I/O port                                  */
outportb(wBase,1);                                /* /RESET -> 1  */

/* step3 : configure I/O direction                                */
outportb(wBase+0xcc,0x00);                /* set CN1 as D/I ports */
outportb(wBase+0xdc,0x00);                /* set CN2 as D/I ports */
outportb(wBase+0xec,0x00);                /* set CN3 as D/I ports */
outportb(wBase+0xfc,0x00);                /* set CN4 as D/I ports */

init();
printf("\n***** show the count of pulse *****\n");

for (;;)
    {
    gotoxy(1,7);
    printf("\n(CNT_L,CNT_H)=(%d,%d) (%d,%d) (%d,%d) (%d,%d) %x",CNT_L1,CNT_H1,CNT_L2,CNT_H2,CNT_L3,CNT_H3,CNT_L4,CNT_H4,int_num);
    if (kbhit()!=0) break;
    }

outportb(wBase+5,0);                     /* disable all interrupt */
PIO_DriverClose();
}

/* Use P2C0, P5C0, P8C0 & P11C0 as external interrupt signal      */
WORD init()
{
disable();
outportb(wBase+5,0);                     /* disable all interrupt */
if (wIrq<8)
   {
   irqmask=inportb(A1_8259+1);
   outportb(A1_8259+1,irqmask & 0xff ^ (1<<wIrq));
   setvect(wIrq+8,irq_service);
   }
else
   {
   irqmask=inportb(A1_8259+1);
   outportb(A1_8259+1,irqmask & 0xfb);                    /* IRQ2 */
   irqmask=inportb(A2_8259+1);
   outportb(A2_8259+1,irqmask & 0xff ^ (1<<(wIrq-8)));
   setvect(wIrq-8+0x70,irq_service);
   }

invert=0x05;
outportb(wBase+0x2a,invert);         /* P2C0  = non-inverte input */
                                     /* P5C0  =     inverte input */
                                     /* P8C0  = non-inverte input */
                                     /* P11C0 =     inverte input */

now_int_state=0x0a;                  /* P2C0  = Low               */
                                     /* P5C0  = High              */
                                     /* P8C0  = Low               */
                                     /* P11C0 = High              */

CNT_L1=CNT_L2=CNT_L3=CNT_L4=0;       /* Low_pulse counter         */
CNT_H1=CNT_H2=CNT_H3=CNT_H4=0;       /* High_pulse counter        */
int_num=0;
outportb(wBase+5,0x0f);              /* enable interrupt P2C0     */
enable();                            /* P5C0, P8C0, P11C0         */
}

void interrupt irq_service()
{
int_num++;
new_int_state=inportb(wBase+7)&0x0f;
int_c=new_int_state^now_int_state;

if ((int_c&0x1)!=0)              /* now INT_CHAN_0 change to high */
   {
   if ((new_int_state&0x01)!=0)
      {
      CNT_H1++;
      }
   else                          /* now INT_CHAN_0 change to low  */
      {
      CNT_L1++;
      }
   invert=invert^1;              /* generate a high pulse         */
   }

if ((int_c&0x2)!=0)              /* now INT_CHAN_1 change to high */
   {
   if ((new_int_state&0x02)!=0)
      {
      CNT_H2++;
      }
   else                          /* now INT_CHAN_1 change to low  */
      {
      CNT_L2++;
      }
   invert=invert^2;              /* generate a high pulse         */
   }

if ((int_c&0x4)!=0)              /* now INT_CHAN_2 change to high */
   {
   if ((new_int_state&0x04)!=0)
      {
      CNT_H3++;
      }
   else                          /* now INT_CHAN_2 change to low  */
      {
      CNT_L3++;
      }
   invert=invert^4;              /* generate a high pulse         */
   }

if ((int_c&0x8)!=0)              /* now INT_CHAN_3 change to high */
   {
   if ((new_int_state&0x08)!=0)
      {
      CNT_H4++;
      }
   else                          /* now INT_CHAN_3 change to low  */
      {
      CNT_L4++;
      }
   invert=invert^8;              /* generate a high pulse         */
   }

now_int_state=new_int_state;
outportb(wBase+0x2a,invert);

if (wIrq>=8) outportb(A2_8259,0x20);
outportb(A1_8259,0x20);
}
