/* DEMO3.C : Use external int. to measure pulse width(high level) */
/* step 1  : a.Clock source select 1MHz (JP1 P3,P4 short)         */
/*           b.Connect P4.1 to CLK2(P3.1)                         */
/*             (Counter2 base on 1MHz clock source)               */
/*             Max. measurement high level width=                 */
/*             (2^16)*(1/1MHz)=0.065535 sec                       */
/*           c.Connect GATE2(P3.2) to GATE4(P5.3)                 */
/*           d.connect GATE2 to waveform generator,and make sure  */
/*             the initial state of the waveform is low-level     */
/* step 2  : run DEMO3.EXE                                        */
/* -------------------------------------------------------------- */
#include "PIO.H"

#define A1_8259 0x20
#define A2_8259 0xA0
#define EOI     0x20

WORD init_low();
WORD pio_d64_c2(char cConfig, char cLow, char cHigh);

       void interrupt *oldfunc; //();
static void interrupt irq_service();      

WORD wBase,wIrq;
int  irqmask,now_int_state,new_int_state,invert,int_c,int_num;
int  CNT_L1,CNT_L2,CNT_L3,CNT_L4;
int  CNT_H1,CNT_H2,CNT_H3,CNT_H4;
unsigned counter;


/* -------------------------------------------------------------- */
WORD init_low()
{
DWORD dwVal;

_disable();
outp(wBase+5,0);                     /* disable all interrupt */

if (wIrq<8)
   {
   oldfunc=_dos_getvect(wIrq+8);
   irqmask=inp(A1_8259+1);
   outp(A1_8259+1,irqmask & (0xff ^ (1 << wIrq)));
   _dos_setvect(wIrq+8, irq_service);
   }
else
   {
   oldfunc=_dos_getvect(wIrq-8+0x70);
   irqmask=inp(A1_8259+1);
   outp(A1_8259+1,irqmask & 0xfb);                    /* IRQ2 */
   irqmask=inp(A2_8259+1);
   outp(A2_8259+1,irqmask & (0xff ^ (1 << (wIrq-8))));
   _dos_setvect(wIrq-8+0x70, irq_service);
   }

pio_d64_c2(0xb4,0xff,0xff);            /* U4.COUNTER2,MODE-2      */

outp(wBase+0x2a,1);                /* non-invert EXTIRQ       */

int_num=0;
CNT_L1=CNT_L2=CNT_L3=CNT_L4=0;         /* Low_pulse counter       */
CNT_H1=CNT_H2=CNT_H3=CNT_H4=0;         /* High_pulse counter      */
now_int_state=0x0;                     /* now EXTIRQ is low       */
outp(wBase+5,0x1);                 /* enable EXTIRQ interrupt */
_enable();
}

/* -------------------------------------------------------------- */
void interrupt irq_service()
{
int_num++;
new_int_state=inp(wBase+7)&0x0f;
int_c=new_int_state^now_int_state;

if ((int_c&0x1)!=0)              /* now INT_CHAN_0 change to high */
   {
   if ((new_int_state&0x01)!=0)
      {
      CNT_H1++;
      }
   else                          /* now INT_CHAN_0 change to low  */
      {
      CNT_L1++;
      outp(wBase+0xdc,0x80);     /* COUNTER2 latch                */
      counter=inp(wBase+0xd8);
      counter=counter|(inp(wBase+0xd8)<<8);
      pio_d64_c2(0xb4,0xff,0xff);/* U4.COUNTER2,MODE-2, reload    */
      }
   invert=invert^1;              /* generate a high pulse         */
   }

if ((int_c&0x2)!=0)              /* now INT_CHAN_1 change to high */
   {
   if ((new_int_state&0x02)!=0)
      {
      CNT_H2++;
      }
   else                          /* now INT_CHAN_1 change to low  */
      {
      CNT_L2++;
      }
   invert=invert^2;              /* generate a high pulse         */
   }

if ((int_c&0x4)!=0)              /* now INT_CHAN_2 change to high */
   {
   if ((new_int_state&0x04)!=0)
      {
      CNT_H3++;
      }
   else                          /* now INT_CHAN_2 change to low  */
      {
      CNT_L3++;
      }
   invert=invert^4;              /* generate a high pulse         */
   }

if ((int_c&0x8)!=0)              /* now INT_CHAN_3 change to high */
   {
   if ((new_int_state&0x08)!=0)
      {
      CNT_H4++;
      }
   else                          /* now INT_CHAN_3 change to low  */
      {
      CNT_L4++;
      }
   invert=invert^8;              /* generate a high pulse         */
   }

now_int_state=new_int_state;
outp(wBase+0x2a,invert);

if (wIrq>=8) outp(A2_8259,0x20);
outp(A1_8259,0x20);
}
/* -------------------------------------------------------------- */
void pio_d64_CN1_do(long lDoValue)
{
outp(wBase+0xc0,(lDoValue&0xff));
outp(wBase+0xc4,((lDoValue>>8)&0xff));
}

/* -------------------------------------------------------------- */
void pio_d64_CN3_do(long lDoValue)
{
outp(wBase+0xc8,(lDoValue&0xff));
outp(wBase+0xcc,((lDoValue>>8)&0xff));
}

/* -------------------------------------------------------------- */
long pio_d64_CN2_di(void)
{
long lDiValue;
lDiValue=(inp(wBase+0xc4)<<8);
lDiValue=(lDiValue|(inp(wBase+0xc0)))&0xffff;
return(lDiValue);
}

/* -------------------------------------------------------------- */
long pio_d64_CN4_di(void)
{
long lDiValue;
lDiValue=(inp(wBase+0xcc)<<8);
lDiValue=(lDiValue|(inp(wBase+0xc8))&0xffff);
return(lDiValue);
}

/* -------------------------------------------------------------- */
WORD pio_d64_c2(char cConfig, char cLow, char cHigh) /* COUNTER_0 */
{
outp(wBase+0xdc,cConfig);
outp(wBase+0xd8,cLow);
outp(wBase+0xd8,cHigh);
return(NoError);
}

int main()
{
int  i,j,k1,k2,l1,l2,jj,dd,j1,i1,j2,i2;
WORD wBoards,wRetVal,t1,t2,t3,t4,t5;
WORD wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice;
long lOutPad1,lOutPad2,lInPad1,lInPad2;
char c;
float width;

_clearscreen(0);
/* step 1: find address-mapping of PIO/PISO cards                 */
wRetVal=PIO_DriverInit(&wBoards,0x80,0x01,0x20);   /* for PIO-D64 */
printf("\nThrer are %d PIO_D64 Cards in this PC",wBoards);
if (wBoards==0) exit(0);

printf("\n--------------- The Configuration Space ---------------");
for(i=0; i<wBoards; i++)
   {
   PIO_GetConfigAddressSpace(i,&wBase,&wIrq,&wSubVendor,&wSubDevice,
   &wSubAux,&wSlotBus,&wSlotDevice);
   printf("\nCard_%d: wBase=%x,wIrq=%x,subID=[%x,%x,%x],SlotID=[%x,%x]",i,wBase,wIrq,wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice);
   printf(" --> ");
   ShowPioPiso(wSubVendor,wSubDevice,wSubAux);
   }

/* select card_0 */
PIO_GetConfigAddressSpace(0,&wBase,&wIrq,&t1,&t2,&t3,&t4,&t5);

/* step 2: enable all D/I/O port                                  */
outp(wBase,1);                                    /* enable D/I/O */
counter=65535;
init_low();

printf("\n\n");
for(;;)
   {
   width=(float)(65535-counter)/1000000.0;
   printf("\rHigh Level width = %f (sec)",width);
   if (kbhit()!=0) break;
   }

_disable();

outp(wBase+5,0);                     /* disable all interrupt */
if (wIrq<8)
   {
   _dos_setvect(wIrq+8,oldfunc);
   }
else
   {
   _dos_setvect(wIrq-8+0x70,oldfunc);
   }
PIO_DriverClose();
}
