/* demo 5 : Four interrupt source                                 */
/*          PC0  : initial Low  , active High                     */
/*          PC1  : initial High , active Low                      */
/*          PC2  : initial Low  , active High                     */
/*          PC3  : initial High , active Low                      */
/* step 1 : run DEMO5.EXE                                         */
/* -------------------------------------------------------------- */

#include "PIO.H"

#define A1_8259 0x20
#define A2_8259 0xA0
#define EOI     0x20

WORD init();
WORD wBase,wIrq;

static void interrupt irq_service();
int irqmask,now_int_state,new_int_state,invert,int_c,int_num;
int CNT_L1,CNT_L2,CNT_L3,CNT_L4;
int CNT_H1,CNT_H2,CNT_H3,CNT_H4;

int main()
{
int  i,j;
WORD wBoards,wRetVal,t1,t2,t3,t4,t5,t6;
WORD wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice;
char c;

_clearscreen(0);

/* step1 : find address-mapping of PIO/PISO cards                 */
wRetVal=PIO_DriverInit(&wBoards,0x80,0x01,0x40);/* for PIO-D56/D24*/
printf("\n(1) Threr are %d PIO-D56/D24 Cards in this PC",wBoards);
if ( wBoards==0 ) exit(0);

printf("\n\n-------------- The Configuration Space --------------");
for(i=0;i<wBoards;i++)
   {
   PIO_GetConfigAddressSpace(i,&wBase,&wIrq,&wSubVendor,&wSubDevice,&wSubAux,&wSlotBus,&wSlotDevice);

   printf("\nCard_%d: wBase=%x,wIrq=%x,subID=[%x,%x,%x],SlotID=[%x,%x]",i,wBase,wIrq,wSubVendor,wSubDevice,wSubAux,wSlotBus,wSlotDevice);

   printf(" --> ");
   ShowPioPiso(wSubVendor,wSubDevice,wSubAux);
   }

PIO_GetConfigAddressSpace(0,&wBase,&wIrq,&t1,&t2,&t3,&t4,&t5);
                                                 /* select card_0 */
/* step2 : enable all D/I/O port                                  */
outp(wBase,1);                                /* /RESET -> 1  */

/* step3 : configure I/O direction                                */
outp(wBase+0xcc,0x00);               /* set CON3 as D/I ports */

init();
printf("\n***** show the count of pulse *****\n");

for (;;)
    {
    gotoxy(1,7);
    printf("\n(CNT_L,CNT_H)=(%d,%d) (%d,%d) (%d,%d) (%d,%d) %x",CNT_L1,CNT_H1,CNT_L2,CNT_H2,CNT_L3,CNT_H3,CNT_L4,CNT_H4,int_num);
    if (kbhit()!=0) break;
    }

outp(wBase+5,0);                     /* disable all interrupt */
PIO_DriverClose();
}

/* Use PC0, PC1, PC2 & PC3 as external interrupt signal           */
WORD init()
{
_disable();
outp(wBase+5,0);                     /* disable all interrupt */
if (wIrq<8)
   {
   irqmask=inp(A1_8259+1);
   outp(A1_8259+1,irqmask & 0xff ^ (1<<wIrq));
   _dos_setvect(wIrq+8,irq_service);
   }
else
   {
   irqmask=inp(A1_8259+1);
   outp(A1_8259+1,irqmask & 0xfb);                    /* IRQ2 */
   irqmask=inp(A2_8259+1);
   outp(A2_8259+1,irqmask & 0xff ^ (1<<(wIrq-8)));
   _dos_setvect(wIrq-8+0x70,irq_service);
   }

invert=0x05;
outp(wBase+0x2a,invert);         /* PC0   = non-inverte input */
                                     /* PC1   =     inverte input */
                                     /* PC2   = non-inverte input */
                                     /* PC3   =     inverte input */

now_int_state=0x0a;                  /* PC0   = Low               */
                                     /* PC1   = High              */
                                     /* PC2   = Low               */
                                     /* PC3   = High              */

CNT_L1=CNT_L2=CNT_L3=CNT_L4=0;       /* Low_pulse counter         */
CNT_H1=CNT_H2=CNT_H3=CNT_H4=0;       /* High_pulse counter        */
int_num=0;
outp(wBase+5,0x0f);              /* enable interrupt PC0,PC1  */
_enable();                            /* PC2,PC3                   */
}

void interrupt irq_service()
{
char c;

int_num++;
new_int_state=inp(wBase+7)&0x0f;  /* read all interrupt state */
int_c=new_int_state^now_int_state;    /* compare which interrupt  */
                                      /* signal be change         */
if ((int_c&0x1)!=0)                   /* INT_CHAN_0 is active     */
   {
   c=new_int_state&0x01;
   if (c!=0) CNT_H1++;        /* now INT_CHAN_0 is change to high */
   else CNT_L1++;             /* now INT_CHAN_0 is change to low  */
   invert=invert^1;           /* to generate a high pulse         */
   }

if ((int_c&0x2)!=0)
   {
   c=new_int_state&0x02;
   if (c!=0) CNT_H2++;        /* now INT_CHAN_1 is change to high */
   else CNT_L2++;             /* now INT_CHAN_1 is change to low  */
   invert=invert^2;           /* to generate a high pulse         */
   }

if ((int_c&0x4)!=0)
   {
   c=new_int_state&0x04;
   if (c!=0) CNT_H3++;        /* now INT_CHAN_2 is change to high */
   else CNT_L3++;             /* now INT_CHAN_2 is change to low  */
   invert=invert^4;           /* to generate a high pulse         */
   }

if ((int_c&0x8)!=0)
   {
   c=new_int_state&0x08;
   if (c!=0) CNT_H4++;        /* now INT_CHAN_3 is change to high */
   else CNT_L4++;             /* now INT_CHAN_3 is change to low  */
   invert=invert^8;           /* to generate a high pulse         */
   }

now_int_state=new_int_state;
outp(wBase+0x2a,invert);

if (wIrq>=8) outp(A2_8259,0x20);
outp(A1_8259,0x20);
}
